/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with {library}; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package ch.hortis.sonar.db;

import org.apache.derby.drda.NetworkServerControl;

import java.io.File;
import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.net.InetAddress;
import java.util.Properties;

public class DatabaseEmbedder {
  
  private static final Properties DEFAULT_PROPS = getDefaultProperties();

  private NetworkServerControl serverControl = null;
  
  private File dbHome;
  private Properties dbProps;
  private PrintWriter dbLog;
  
  public DatabaseEmbedder( File dbHome ) {
    this.dbHome = dbHome;
    this.dbProps = DEFAULT_PROPS;
  }
  
  public DatabaseEmbedder( File dbHome, Properties dbProps ) {
    this( dbHome );
    this.dbProps = dbProps;
  }

  public void setDbLog( PrintWriter dbLog ) {
    this.dbLog = dbLog;
  }

  public void start() throws Exception {
    if ( dbHome.exists() && !dbHome.isDirectory() ) {
      throw new Exception( "Database home " + dbHome.getPath() + " is not a directory" );
    }
    if ( !dbHome.exists() ) {
      dbHome.mkdirs();
    }
    System.setProperty( "derby.system.home", dbHome.getPath() );
    if ( dbProps != null ) {
      File derbyProps = new File( dbHome.getPath() + "/derby.properties" );
      dbProps.store( new FileOutputStream( derbyProps ), "GENERATED FILE, DO NOT EDIT ME UNLESS YOU WANT TO LOOSE YOUR TIME ;O)" );
    }
    String providedPort = dbProps.getProperty("derby.drda.portNumber");
    if (providedPort == null || providedPort.equals("")) {
      throw new Exception("Missing derby.drda.portNumber property");
    }
    int port = Integer.parseInt(providedPort);
    serverControl = new NetworkServerControl(InetAddress.getByName("localhost"), port);
    serverControl.start(dbLog);
    ensureServerIsUp();
  }
  
  public File getDbHome() {
    return dbHome;
  }
  
  public void stop() throws Exception {
    if ( serverControl != null ) {
      serverControl.shutdown();
      ensureServerIsDown();
      serverControl = null;
    }
  }
  
  private void ensureServerIsUp() throws Exception {
    for ( int retry = 0; retry < 16; retry++ ) {
      try {
        serverControl.ping();
        return;
      } catch ( Exception ex ) {
        sleep( 250 );
      }
    }
    throw new Exception( "Embedded server does not respond to ping requests" );
  }
  
  private void sleep( long time ) {
    try {
      Thread.sleep( time );
    } catch ( InterruptedException e ) {
    }
  }
  
  private void ensureServerIsDown() throws Exception {
    for ( int retry = 0; retry < 16; retry++ ) {
      try {
        serverControl.ping();
        sleep( 250 );
      } catch ( Exception ex ) {
        return;
      }
    }
    throw new Exception( "Embedded server did not stop" );
  }
  
  public static Properties getDefaultProperties() {
    Properties props = new Properties();
    props.setProperty( "derby.drda.startNetworkServer", "true" );
    props.setProperty( "derby.drda.portNumber", "1527" ); // default port
    props.setProperty( "derby.drda.maxThreads", "20" );
    props.setProperty( "derby.drda.minThreads", "2" );
    props.setProperty( "derby.drda.logConnections", "false" );
    props.setProperty( "derby.stream.error.logSeverityLevel", "20000" );
    props.setProperty( "derby.connection.requireAuthentication", "true" );
    props.setProperty( "derby.user.sonar", "sonar" );
    return props;
  }
  
}
